﻿local addon, ns = ...
local cfg = ns.cfg
local tags = ns.tags
local mediaFolder = "Interface\\AddOns\\oUF_Nivea\\media\\"	-- don't touch this ...
local playerClass = select(2, UnitClass("player"))	

local blankTex = "Interface\\Buttons\\WHITE8x8"
local backdrop = {bgFile = blankTex, edgeFile = blankTex, edgeSize = 1, insets = { left = -1, right = -1, top = -1, bottom = -1}}
local backdrop2 = {edgeFile = mediaFolder.."glowtex", edgeSize = 5, insets = {left = 3, right = 3, top = 3, bottom = 3}}

-- change some colors
local colors = setmetatable({
	power = setmetatable({
	["MANA"] = {0.36, 0.45, 0.88},
	["RAGE"] = {0.8, 0.21, 0.31},
	["FUEL"] = {0, 0.55, 0.5},
	["FOCUS"] = {0.71, 0.43, 0.27},
	["ENERGY"] = {0.85, 0.83, 0.35},
	["AMMOSLOT"] = {0.8, 0.6, 0},
	["RUNIC_POWER"] = {0, 0.82, 1},
	["POWER_TYPE_STEAM"] = {0.55, 0.57, 0.61},
	["POWER_TYPE_PYRITE"] = {0.60, 0.09, 0.17},
	["HOLY_POWER"] = {0.95, 0.93, 0.15},	
	["SOUL_SHARDS"] = {0.5, 0.0, 0.56},
	}, {__index = oUF.colors.power}),
}, {__index = oUF.colors})

-- format numbers
function round(num, idp)
  if idp and idp > 0 then
    local mult = 10^idp
    return math.floor(num * mult + 0.5) / mult
  end
  return math.floor(num + 0.5)
end

function CoolNumber(num)
	if(num >= 1e6) then
		return round(num/1e6,cfg.Numberzzz).."m"
	elseif(num >= 1e3) then
		return round(num/1e3,cfg.Numberzzz).."k"
	else
		return num
	end
end

-- health update
local PostUpdateHealth = function(Health, unit, min, max)
	local self = Health:GetParent()
    local d =(round(min/max, 2)*100)
	local c = UnitClassification(unit)
	
	if(UnitIsDead(unit)) then
		Health:SetValue(0)
		Health.value:SetText"RIP"			
	elseif(UnitIsGhost(unit)) then
		Health:SetValue(0)
		Health.value:SetText"GHO"			
	elseif(not UnitIsConnected(unit)) then	
		Health.value:SetText"OFF"	
	elseif(c == 'worldboss') then
		Health.value:SetText(CoolNumber(min).."  "..(round(min/max, 2)*100))	
	else
		Health.value:SetText(CoolNumber(min))	
		Health:SetStatusBarColor(unpack(cfg.maincolor))		
	end

	-- set health background color
	if not cfg.solidHPBGcolor then
		local color = RAID_CLASS_COLORS[select(2, UnitClass(unit))]	
		if UnitIsPlayer(unit) and color then
			self.Health.bg:SetVertexColor(color.r, color.g, color.b, 0.8)
		else
			local r, g, b = UnitSelectionColor(unit)
			self.Health.bg:SetVertexColor(r, g, b, 0.8)	
		end
		
		if UnitLevel(unit) == 0 then
			self.Health.bg:SetVertexColor(0, 0, 0, 0.5)
		elseif UnitIsConnected(unit) ~= 1 then
			self.Health.bg:SetVertexColor(0, 0, 0, 0.5)
		elseif UnitIsDeadOrGhost(unit) == 1 then
			self.Health.bg:SetVertexColor(200/255, 20/255, 40/255, 0.5)
		end
	else
		if UnitIsDeadOrGhost(unit) == 1 then
			self.Health.bg:SetVertexColor(200/255, 20/255, 40/255, 0.5)
		else
			hpBGr, hpBGg, hpBGb = unpack(cfg.hpBGcolor)
			self.Health.bg:SetVertexColor(hpBGr, hpBGg, hpBGb, 1)
		end
	end		
	
	-- set text color
	if(unit) then
		if(d <= 35 and d >= 25) then
			Health.value:SetTextColor(253/255, 238/255, 80/255)
		elseif(d < 25 and d >= 20) then
			Health.value:SetTextColor(250/255, 130/255, 0/255)
		elseif(d < 20) then
			Health.value:SetTextColor(200/255, 20/255, 40/255)
		else
			Health.value:SetTextColor(unpack(cfg.sndcolor))
		end
	end	
end

-- power update
local PostUpdatePower = function(Power, unit, min, max)
	local dp =(round(min/max, 2)*100)
	
	if(min == 0 or max == 0 or not UnitIsConnected(unit)) then
		Power.value:SetText()
		Power:SetValue(0)
	elseif(UnitIsDead(unit) or UnitIsGhost(unit)) then
		Power:SetValue(0)
		Power.value:SetText()
	elseif(unit == "player") then
		if(dp < 100) then
			Power.value:Show()
			Power.value:SetText(CoolNumber(min))
		else
			Power.value:Hide()
		end	
	else
		Power.value:SetText(CoolNumber(min))
	end
	
	-- color power text by power type
	local _, ptype = UnitPowerType(unit)
        if(colors.power[ptype]) then
		r, g, b = unpack(colors.power[ptype])
	end
	
	Power.value:SetTextColor(r, g, b)
end

local PostUpdatePowerRaid = function(Power, unit)
	local powertype, _ = UnitPowerType(unit)
	if powertype ~= SPELL_POWER_MANA then
		Power:Hide()
	else
		Power:Show()
	end	
end	
	
-- custom castbar text (curCastTime/maxCastTime)
local function CustomTimeText(self, duration)
	if self.casting then
		self.Time:SetFormattedText('%.1f /', (self.max - duration))
		self.Time2:SetFormattedText(' %.1f', self.max)
	elseif self.channeling then
		self.Time:SetFormattedText('%.1f /', duration)
		self.Time2:SetFormattedText(' %.1f', self.max)
	end
end

--------------------
-- aura functions --
--------------------
-- filter some crap
local Whitelist = Whitelist.auras
local CustomFilter = function(icons, unit, icon, name, rank, texture, count, dtype, duration, timeLeft, caster, isStealable, shouldConsolidate, spellID)
	if(Whitelist[name]) then
		return true
	end 
end

-- format time
local FormatTime = function(s)
	local day, hour, minute = 86400, 3600, 60
	if s >= day then
		return format("%dd", floor(s/day + 0.5)), s % day
	elseif s >= hour then
		return format("%dh", floor(s/hour + 0.5)), s % hour
	elseif s >= minute then
		if s <= minute * 5 then
			return format('%d:%02d', floor(s/60), s % minute), s - floor(s)
		end
		return format("%dm", floor(s/minute + 0.5)), s % minute
	elseif s >= minute / 12 then
		return floor(s + 0.5), (s * 100 - floor(s * 100))/100
	end
	return format("%.1f", s), (s * 100 - floor(s * 100))/100
end


-- aura timer 
local CreateAuraTimer = function(self, elapsed)
		self.elapsed = (self.elapsed or 0) + elapsed
	if self.elapsed >= 0.1 then
		if not self.first then
			self.timeLeft = self.timeLeft - self.elapsed
		else
			self.timeLeft = self.timeLeft - GetTime()
			self.first = false
		end
		if self.timeLeft > 0 then
			local time = FormatTime(self.timeLeft)
			self.time:SetText(time)
			if self.timeLeft < 5 then
				self.time:SetTextColor(1, 0, 0)
			elseif self.timeLeft >= 5 and self.timeLeft < cfg.HideAuraTimer then
				self.time:SetTextColor(unpack(cfg.sndcolor))
			else
				self.time:SetText('')
				self.time:SetTextColor(unpack(cfg.sndcolor))			
			end
			else
				self.time:Hide()
				self:SetScript("OnUpdate", nil)
			end
			self.elapsed = 0
		end
end

-- icon style
local PostCreateIcon = function(Auras, button)
	local buttonwidth = button:GetWidth()
	button.cd.noOCC = true		 		-- hide OmniCC CDs
	button.cd.noCooldownCount = true	-- hide CDC CDs
	Auras.disableCooldown = true		-- hide CD spiral
	Auras.showDebuffType = true			-- show debuff border type color 		
	
	button.overlay:SetTexture(cfg.Auratex)
    button.overlay:SetPoint("TOPLEFT", button.icon, "TOPLEFT", -2, 2)
    button.overlay:SetPoint("BOTTOMRIGHT", button.icon, "BOTTOMRIGHT", 2, -2)		
	button.overlay:SetTexCoord(0, 1, 0, 1)
	button.overlay.Hide = function(self) self:SetVertexColor(unpack(cfg.brdcolor)) end		
	
	button.time = button:CreateFontString(nil, 'OVERLAY')
	button.time:SetFont(cfg.NumbFont, buttonwidth/2.6, cfg.fontFNum)	
	button.time:SetPoint("BOTTOM", button, 0, -2)
	button.time:SetJustifyH('CENTER')
	button.time:SetVertexColor(unpack(cfg.sndcolor))	
	
	--[[	
	button.Glow = CreateFrame("Frame", nil, button)
	button.Glow:SetPoint("TOPLEFT", button, "TOPLEFT", -6, 6)
	button.Glow:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 6, -6)
	button.Glow:SetBackdrop(backdrop2)
	button.Glow:SetBackdropColor(0, 0, 0)
	button.Glow:SetBackdropBorderColor(0, 0, 0)
	button.Glow:SetFrameLevel(0)	
	--]]
	local count = button.count
	count:ClearAllPoints()
	count:SetPoint("TOPRIGHT", button, 1, 1)
	count:SetFont(cfg.NumbFont, buttonwidth/2.6, cfg.fontFNum)
	count:SetVertexColor(unpack(cfg.sndcolor))
	
	button.icon:SetTexCoord(.07, .93, .07, .93)
end

-- weapon enchant icon
local function WeapEnchantIcon(self, icon, icons)
	icon.time = icon:CreateFontString(nil, 'OVERLAY')
	icon.time:SetFont(cfg.NumbFont, cfg.NumbFS, cfg.fontFNum)
	icon.time:SetPoint("BOTTOM", icon, 0, -2)
	icon.time:SetJustifyH('CENTER')
	icon.time:SetVertexColor(unpack(cfg.sndcolor))
	
	icon.overlay:SetTexture(cfg.Auratex)
	icon.overlay:SetTexCoord(0, 1, 0, 1)
	icon.overlay:SetVertexColor(unpack(cfg.brdcolor))	
end

local CreateEnchantTimer = function(self, icons)
	for i = 1, 2 do
		local icon = icons[i]
		if icon.expTime then
			icon.timeLeft = icon.expTime - GetTime()
			icon.time:Show()
		else
			icon.time:Hide()
		end
		icon:SetScript("OnUpdate", CreateAuraTimer)
	end
end

-- update icon
local PostUpdateIcon
do
	local playerUnits = {
		player = true,
		pet = true,
		vehicle = true,
	}

	PostUpdateIcon = function(icons, unit, icon, index, offset, filter, isDebuff)
	local name, _, _, _, dtype, duration, expirationTime, unitCaster, _ = UnitAura(unit, index, icon.filter)
	local texture = icon.icon
	
	if unit == "target" or unit == "focus" then
		if duration and duration > 0 and Whitelist[name] or (playerUnits[icon.owner]) then
			icon.time:Show()
			icon.timeLeft = expirationTime
			icon:SetScript("OnUpdate", CreateAuraTimer)
		else
			icon.time:Hide()
		end
	else
		if duration and duration > 0 then
			icon.time:Show()
			icon.timeLeft = expirationTime
			icon:SetScript("OnUpdate", CreateAuraTimer)
		else
			icon.time:Hide()
		end
	end

	if unit == "target" or unit == "focus" then
		if (playerUnits[icon.owner]) then
			if dtype == "Magic" or dtype == "Disease" or dtype == "Poison" or dtype == "Curse" then
				local color = DebuffTypeColor[dtype] or DebuffTypeColor.none
				icon.overlay:SetVertexColor(color.r*0.9, color.g*0.9, color.b*0.9)
				texture:SetDesaturated(false)
			end
		else
			icons.showDebuffType = true
			icon.overlay:SetVertexColor(unpack(cfg.brdcolor))	
			texture:SetDesaturated(true)
		end	
	end	
	
	icon.first = true
	end
end

-- right click menu
local dropdown = CreateFrame("Frame", "MyAddOnUnitDropDownMenu", UIParent, "UIDropDownMenuTemplate")
UIDropDownMenu_Initialize(dropdown, function(self)
	local unit = self:GetParent().unit
	if not unit then return end

	local menu, name, id
	if UnitIsUnit(unit, "player") then
		menu = "SELF"
	elseif UnitIsUnit(unit, "vehicle") then
		menu = "VEHICLE"
	elseif UnitIsUnit(unit, "pet") then
		menu = "PET"
	elseif UnitIsPlayer(unit) then
		id = UnitInRaid(unit)
		if id then
			menu = "RAID_PLAYER"
			name = GetRaidRosterInfo(id)
		elseif UnitInParty(unit) then
			menu = "PARTY"
		else
			menu = "PLAYER"
		end
	else
		menu = "TARGET"
		name = RAID_TARGET_ICON
	end
	if menu then
		UnitPopup_ShowMenu(self, menu, unit, name, id)
	end
end, "MENU")

local menu = function(self)
	dropdown:SetParent(self)
	ToggleDropDownMenu(1, nil, dropdown, "cursor", 0, 0)
end

-- threat highlight
local function updateThreatStatus(self, event, u)
	if (self.unit ~= u) then return end
	local s = UnitThreatSituation(u)
	if s and s > 1 then
		local r, g, b = GetThreatStatusColor(s)
		self.ThreatHlt:Show()
		self.ThreatHlt:SetVertexColor(r, g, b, 0.5)
	else
		self.ThreatHlt:Hide()
	end
end

-- debuff highlight
local CanDispel = {
	PRIEST = { Magic = true, Disease = true, },
	SHAMAN = { Magic = true, Curse = true},
	PALADIN = { Magic = true, Poison = true, Disease = true, },
	MAGE = { Curse = true, },
	DRUID = { Magic = true, Curse = true, Poison = true},
}
local dispellist = CanDispel[playerClass] or {}

local function GetDebuffType(unit)
	if not UnitCanAssist("player", unit) then return nil end
	local i = 1
	while true do
		local name, rank, texture, count, debufftype, duration, expirationTime, source = UnitDebuff(unit, i)
		if not texture then break end
		if debufftype and dispellist[debufftype] then
			return debufftype
		end
		i = i + 1
	end
end

local function updateDispel(self, event, u)
	if not u or self.unit ~= u then return end
	local debufftype = GetDebuffType(u)
	if debufftype then
		local color = DebuffTypeColor[debufftype] 
		self.InfoPan:SetVertexColor(color.r, color.g, color.b, 0.7)	
	else
		local br, bg, bb = unpack(cfg.infocolor)
		self.InfoPan:SetVertexColor(br, bg, bb, 1)		
	end
end

-- mouseover highlight
local UnitFrame_OnEnter = function(self)
	UnitFrame_OnEnter(self)
	self.Mouseover:Show()	
end

local UnitFrame_OnLeave = function(self)
	UnitFrame_OnLeave(self)
	self.Mouseover:Hide()
end

--------------------------------
-- shared stuff for all units --
--------------------------------
local Shared = function(self, unit, isSingle)
local playerClass = select(2, UnitClass("player"))	
local unitIsPartyPet = self:GetAttribute("unitsuffix") == "pet"

	self.menu = menu
	self:SetAttribute("*type2", "menu")

	self:SetScript("OnEnter", UnitFrame_OnEnter)
	self:SetScript("OnLeave", UnitFrame_OnLeave)

	self:RegisterForClicks"AnyUp"

	-- set/clear focus with shift + left click
    local ModKey = 'Shift'
    local MouseButton = 1
    local key = ModKey .. '-type' .. (MouseButton or '')
	if(self.unit == 'focus') then
		self:SetAttribute(key, 'macro')
		self:SetAttribute('macrotext', '/clearfocus')
	else
		self:SetAttribute(key, 'focus')
	end
	
	if cfg.HideBlizzardAuras == true then
		-- hide blizzard buff, debuff and weapon enchant frame
		local BlizzFrame = _G['BuffFrame']
		BlizzFrame:UnregisterEvent('UNIT_AURA')
		BlizzFrame:Hide()
		BlizzFrame = _G['TemporaryEnchantFrame']
		BlizzFrame:Hide()
	end
	
	-- hp
	local hp = CreateFrame("StatusBar", nil, self)
	hp:SetHeight(cfg.heightHP)	
	hp:SetStatusBarTexture(cfg.HPtex)
	hp:SetPoint"TOP"
	hp:SetPoint"LEFT"
	hp:SetPoint"RIGHT"
	hp:SetStatusBarColor(unpack(cfg.maincolor))
	hp:GetStatusBarTexture():SetHorizTile(true)
	
	hp.frequentUpdates = true
	self.Health = hp
	self.Health.PostUpdate = PostUpdateHealth	
	
	-- hp border
	self.Glow = CreateFrame("Frame", nil, hp)
	self.Glow:SetPoint("TOPLEFT", hp, "TOPLEFT", -2, 2)
	self.Glow:SetPoint("BOTTOMRIGHT", hp, "BOTTOMRIGHT", 2, -2)
	self.Glow:SetBackdrop(backdrop)
	self.Glow:SetBackdropColor(unpack(cfg.backdropcolor))
	self.Glow:SetBackdropBorderColor(unpack(cfg.brdcolor))	
	self.Glow:SetFrameLevel(1)
	
	-- hp bg
	hpbg = hp:CreateTexture(nil, "BACKGROUND")
	hpbg:SetAllPoints(hp)
	hpbg:SetTexture(cfg.Itex)
	hp.bg = hpbg
	
	-- pp
	local pp = CreateFrame("StatusBar", nil, self)
	pp:SetWidth(cfg.widthP)			
	pp:SetHeight(cfg.heightPP)
	pp:SetStatusBarTexture(cfg.PPtex)
	pp:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, cfg.PPyOffset)
	pp:GetStatusBarTexture():SetHorizTile(true)
	
	pp.frequentUpdates = true
	pp.colorPower = true	
	self.Power = pp	
	self.Power.PostUpdate = PostUpdatePower	
	
	-- pp border
	self.Glow.pp = CreateFrame("Frame", nil, pp)
	self.Glow.pp:SetPoint("TOPLEFT", pp, "TOPLEFT", -2, 2)
	self.Glow.pp:SetPoint("BOTTOMRIGHT", pp, "BOTTOMRIGHT", 2, -2)
	self.Glow.pp:SetBackdrop(backdrop)
	self.Glow.pp:SetBackdropColor(unpack(cfg.backdropcolor))
	self.Glow.pp:SetBackdropBorderColor(unpack(cfg.brdcolor))	
	self.Glow.pp:SetFrameLevel(1)	
	
	-- pp bg
	local ppBG = pp:CreateTexture(nil, 'BORDER')
	ppBG:SetAllPoints()	
	ppBG:SetTexture(cfg.Itex)
	ppBG:SetAlpha(0.8)	
	ppBG.multiplier = 0.4
	pp.bg = ppBG	
	
	-- enable custom colors
	self.colors = colors

	-- font strings
	self.Health.value = hp:CreateFontString(nil, "OVERLAY")
	self.Health.value:SetFont(cfg.NumbFont, cfg.NumbFS, cfg.fontFNum)
	
	self.Power.value = pp:CreateFontString(nil, "OVERLAY")
	self.Power.value:SetFont(cfg.NumbFont, cfg.NumbFS, cfg.fontFNum)	
	
	self.Name = hp:CreateFontString(nil, "OVERLAY")
	self.Name:SetFont(cfg.NameFont, cfg.NameFS, cfg.FontF)
	self:Tag(self.Name, '[raidcolor][abbrevname]')

	self.Status = hp:CreateFontString(nil, "OVERLAY")
	self.Status:SetFont(cfg.NameFont, cfg.NameFS, cfg.FontF)
	self:Tag(self.Status, '[afkdnd][difficulty][smartlevel]')
	
	-- mouseover highlight
	local mov = self.Health:CreateTexture(nil, "OVERLAY")
	mov:SetAllPoints(self.Health)
	mov:SetTexture("Interface\\AddOns\\oUF_Nivea\\media\\mouseover")
	mov:SetVertexColor(1,1,1,.36)
	mov:SetBlendMode("ADD")
	mov:Hide()
	self.Mouseover = mov
	
	-- threat highlight
	local Thrt = self.Health:CreateTexture(nil, "OVERLAY")
	Thrt:SetAllPoints(self.Health)
	Thrt:SetTexture("Interface\\AddOns\\oUF_Nivea\\media\\threat")
	--Thrt:SetVertexColor(1,1,1,.36)
	Thrt:SetBlendMode("ADD")
	Thrt:Hide()
	self.ThreatHlt = Thrt	
	
	-- update threat
	self:RegisterEvent("UNIT_THREAT_LIST_UPDATE", updateThreatStatus)
	self:RegisterEvent("UNIT_THREAT_SITUATION_UPDATE", updateThreatStatus)
	
	-- info panel
	self.Info = CreateFrame("Frame", nil, self)
	self.Info:SetHeight(cfg.NumbFS+4)
	self.Info:SetPoint"TOP"
	self.Info:SetPoint"LEFT"
	self.Info:SetPoint"RIGHT"	
	self.Info:SetPoint("TOP", self.Health, "BOTTOM", 0, -cfg.InfoOffset)

	self.InfoPan = self.Info:CreateTexture(nil,"BACKGROUND")
	self.InfoPan:SetTexture(cfg.Itex)
	self.InfoPan:SetAllPoints(self.Info)	
	self.InfoPan:SetVertexColor(unpack(cfg.infocolor))		
	
	-- info panel border	
	self.InfoBG = CreateFrame("Frame", nil, self)
	self.InfoBG:SetPoint("TOPLEFT", self.Info, "TOPLEFT", -2, 2)
	self.InfoBG:SetPoint("BOTTOMRIGHT", self.Info, "BOTTOMRIGHT", 2, -2)	
	self.InfoBG:SetBackdrop(backdrop)	
	self.InfoBG:SetBackdropColor(unpack(cfg.backdropcolor))
	self.InfoBG:SetBackdropBorderColor(unpack(cfg.brdcolor))	
	self.InfoBG:SetFrameLevel(1)

	self.FrameGlow = CreateFrame("Frame", nil, self)
	--self.FrameGlow:SetPoint("TOPLEFT", self, "TOPLEFT", -2, 2)
	--self.FrameGlow:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", 2, -2)
	self.FrameGlow:SetBackdrop(backdrop2)
	self.FrameGlow:SetBackdropColor(0, 0, 0)
	self.FrameGlow:SetBackdropBorderColor(0, 0, 0)
	self.FrameGlow:SetFrameLevel(0)	
end

----------------------
-- object functions --
----------------------
	
-- create and update portraits
local Portrait_PostUpdate = function(Portrait, unit)
	if (not UnitExists(unit) or not UnitIsConnected(unit) or not UnitIsVisible(unit)) then
		Portrait:Hide()
	else
		Portrait:Show()
		Portrait:SetCamera(0)
		-- workaround for setting portrait alpha ...
		Portrait:SetAlpha(0)
		Portrait:SetAlpha(cfg.pAlpha)
	end
end

local createPortraits = function(self, unit)
	self.Portrait = CreateFrame("PlayerModel", nil, self.Health)
	self.Portrait:SetAllPoints(self.Health)
end

local createPortraitBGBorder = function(self, unit)
	local PortraitBG = self:CreateTexture(nil, "BACKGROUND")
	PortraitBG:SetTexture(cfg.Itex)
	PortraitBG:SetAllPoints(self.Portrait)	
	PortraitBG:SetVertexColor(unpack(cfg.infocolor))		
	PortraitBG:SetAlpha(1)
	
	self.PortraitBorder = CreateFrame("Frame", nil, self)
	self.PortraitBorder:SetPoint("TOPLEFT", self.Portrait, "TOPLEFT", -2, 2)
	self.PortraitBorder:SetPoint("BOTTOMRIGHT", self.Portrait, "BOTTOMRIGHT", 2, -2)	
	self.PortraitBorder:SetBackdrop(backdrop)
	self.PortraitBorder:SetBackdropColor(unpack(cfg.backdropcolor))
	self.PortraitBorder:SetBackdropBorderColor(unpack(cfg.brdcolor))	
	self.PortraitBorder:SetFrameLevel(0)		
end	

-- castbar
local createCastbar = function(self, unit)
	local cb = CreateFrame("StatusBar", nil, self)
	cb:SetStatusBarTexture(cfg.CBtex)
	cb:SetStatusBarColor(unpack(cfg.trdcolor))
	cb:SetFrameLevel(8)

	self.Castbar = cb

	cb.Text = cb:CreateFontString(nil, 'ARTWORK')
	cb.Text:SetJustifyH("LEFT")		
	cb.Text:SetFont(cfg.NameFont, cfg.CastFS, cfg.FontF)
	cb.Text:SetTextColor(unpack(cfg.sndcolor)) 
		
	cb.Time = cb:CreateFontString(nil, 'ARTWORK')
	cb.Time:SetFont(cfg.NumbFont, cfg.CastFS, cfg.fontFNum)
	cb.Time:SetJustifyH('RIGHT')		
	cb.Time:SetTextColor(unpack(cfg.sndcolor))	

	cb.Time2 = cb:CreateFontString(nil, 'ARTWORK')
	cb.Time2:SetFont(cfg.NumbFont, cfg.CastFS, cfg.fontFNum)
	cb.Time2:SetJustifyH('RIGHT')		
	cb.Time2:SetTextColor(unpack(cfg.sndcolor))
	
	cb.CustomTimeText = CustomTimeText	
	
	if cfg.useSpellIcon then 	
		cb.Icon = cb:CreateTexture(nil, 'OVERLAY')
		cb.Icon:SetHeight(cfg.heightP+10)
		cb.Icon:SetWidth(cfg.heightP+10)
		cb.Icon:SetTexCoord(0.1,0.9,0.1,0.9)
		
		cb.IconGlow = CreateFrame("Frame", nil, cb)
		cb.IconGlow:SetPoint("TOPLEFT", cb.Icon, "TOPLEFT", -2, 2)
		cb.IconGlow:SetPoint("BOTTOMRIGHT", cb.Icon, "BOTTOMRIGHT", 2, -2)
		cb.IconGlow:SetBackdrop(backdrop)
		cb.IconGlow:SetBackdropColor(unpack(cfg.backdropcolor))
		cb.IconGlow:SetBackdropBorderColor(unpack(cfg.brdcolor))	
		cb.IconGlow:SetFrameLevel(0)
	end
		
	cb.bg1 = cb:CreateTexture(nil, "BORDER")
	cb.bg1:SetAllPoints(cb)
	cb.bg1:SetTexture(cfg.Itex)
	cb.bg1:SetVertexColor(0.2,0.2,0.2,1)	
end

-- buffs
local createBuffs = function(self)
	local Buffs = CreateFrame("Frame", nil, self)
	Buffs:SetPoint'LEFT'
	Buffs:SetPoint'RIGHT'
	Buffs.num = 40
	Buffs.spacing = 4	

	self.Buffs = Buffs
	
	Buffs.PostCreateIcon = PostCreateIcon
	Buffs.PostUpdateIcon = PostUpdateIcon	
end

-- debuffs
local createDebuffs = function(self)
	local Debuffs = CreateFrame("Frame", nil, self)
	Debuffs:SetPoint'LEFT'
	Debuffs:SetPoint'RIGHT'
	Debuffs.spacing = 4

	self.Debuffs = Debuffs
	
	Debuffs.PostCreateIcon = PostCreateIcon
	Debuffs.PostUpdateIcon = PostUpdateIcon
end

-- phase icon
local createPhaseIcon = function(self)
	local pIcon = self.Health:CreateTexture(nil, 'OVERLAY')
	pIcon:SetSize(18, 18)
	self.PhaseIcon = pIcon
end

-- quest icon
local createQuestIcon = function(self)
	local qIcon = self.Health:CreateTexture(nil, 'OVERLAY')
	qIcon:SetSize(18, 18)
	self.QuestIcon = qIcon
end

-- raid icons
local createRaidIcon = function(self)
	RI = self.Health:CreateTexture(nil, "OVERLAY")
	RI:SetTexture("Interface\\AddOns\\oUF_Nivea\\media\\raidicons.blp")
	RI:SetSize(18, 18)
	self.RaidIcon = RI
end
		
-- class specific tags
local classTags = function(self)
		local PriestTags = "[NiveaPWS][NiveaRenew][NiveaPoM][NiveaGS][NiveaFW][NiveaWS]"
		local PaladinTags = "[NiveaBoL]"
		local DruidTags = "[NiveaRej][NiveaReg][NiveaLB][NiveaWG][NiveaInn]"
		local ShamanTags = "[NiveaES][NiveaRipT][NiveaErLiv]"	
		
		self.Text = self.Health:CreateFontString(nil, "ARTWORK")
		self.Text:SetFont("Interface\\AddOns\\dMedia\\PointF.ttf", 8, "OUTLINE MONOCHROME")
		self.Text:SetPoint("TOPRIGHT", self.Health, "TOPRIGHT", 3, -1)
		
		if playerClass == "PRIEST" then			
			self:Tag(self.Text, PriestTags)
		elseif playerClass == "PALADIN" then			
			self:Tag(self.Text, PaladinTags)
		elseif playerClass == "DRUID" then			
			self:Tag(self.Text, DruidTags)
		elseif playerClass == "SHAMAN" then
			self:Tag(self.Text, ShamanTags) 			
		else
			self:Tag(self.Text, " ")
		end
end			
	
-- plugin support
local SpellRange = function(self)
	if IsAddOnLoaded("oUF_SpellRange") then	
		self.SpellRange = {
		insideAlpha = 1,
		outsideAlpha = cfg.FadeOutAlpha}	
	end
end

local CombatFeedback = function(self)
	if IsAddOnLoaded("oUF_CombatFeedback") then
		local cbft = self.Health:CreateFontString(nil, "ARTWORK")
		cbft:SetPoint("CENTER", self.Health, "CENTER", 0, 0)
		self.CombatFeedbackText = cbft	
		self.CombatFeedbackText:SetFont(cfg.NumbFont, cfg.NumbFS, cfg.fontFNum)
	end
end
	
local SmoothUpdate = function(self)
	if IsAddOnLoaded("oUF_Smooth") then
		self.Health.Smooth = true
		self.Power.Smooth = true	
	end	
end

local BarFader = function(self)
	if IsAddOnLoaded("oUF_BarFader") then		
		self.BarFade = true
		self.BarFaderMinAlpha = cfg.BarFadeAlpha	
	end
end
	
local HealComm4 = function(self)	
	if IsAddOnLoaded("oUF_HealComm4") then	
		HCB = CreateFrame('StatusBar', nil, self.Health)
		HCB:SetStatusBarTexture(cfg.HPtex)
		HCB:SetStatusBarColor(0, 0.8, 0, 0.5)
		HCB:SetPoint('LEFT', self.Health, 'LEFT')
		self.HealCommBar = HCB
		self.allowHealCommOverflow = false
		self.HealCommOthersOnly = true
	end
end

local WeaponEnchant = function(self)
	if IsAddOnLoaded("oUF_WeaponEnchant") then
		self.Enchant = CreateFrame("Frame", nil, self)
		self.Enchant:SetSize(cfg.WeapEnchantIconSize * 2, cfg.WeapEnchantIconSize)
		self.Enchant:SetPoint("TOPRIGHT", self, 0, 0)
		self.Enchant.size = cfg.WeapEnchantIconSize
		self.Enchant.spacing = 2
		self.Enchant.initialAnchor = "TOPRIGHT"
		self.Enchant["growth-x"] = "LEFT"
		self.Enchant:SetFrameLevel(10)
		self.PostCreateEnchantIcon = WeapEnchantIcon
		self.PostUpdateEnchantIcons = CreateEnchantTimer
	end
end

local TotemBar = function(self)
	if IsAddOnLoaded("oUF_TotemBar") then
		self.TotemBar = {}
		for i = 1, 4 do
			self.TotemBar[i] = CreateFrame("StatusBar", nil, self)
			self.TotemBar[i]:SetHeight(cfg.ClassBarHeight)
			self.TotemBar[i]:SetWidth(((cfg.widthP - 9)/4))
		if (i == 1) then
			self.TotemBar[i]:SetPoint("TOPLEFT", self.Info, "BOTTOMLEFT", 0, -3)
		else
			self.TotemBar[i]:SetPoint("TOPLEFT", self.TotemBar[i-1], "TOPRIGHT", 3, 0)
		end
			self.TotemBar[i]:SetStatusBarTexture(cfg.Itex)
			self.TotemBar[i]:SetMinMaxValues(0, 1)
			self.TotemBar[i].destroy = true			
			
			self.TotemBar[i].bg = self.TotemBar[i]:CreateTexture(nil, "BORDER")
			self.TotemBar[i].bg:SetAllPoints(self.TotemBar[i])
			self.TotemBar[i].bg:SetTexture(cfg.Itex)
			self.TotemBar[i].bg.multiplier = 0.4
			self.TotemBar[i].bg:SetAlpha(1.0)

			self.TotemBarBorder = CreateFrame("Frame", nil, self)
			self.TotemBarBorder:SetPoint("TOPLEFT", self.TotemBar[i], "TOPLEFT", -2, 2)
			self.TotemBarBorder:SetPoint("BOTTOMRIGHT", self.TotemBar[i], "BOTTOMRIGHT", 2, -2)					
			self.TotemBarBorder:SetBackdrop(backdrop)
			self.TotemBarBorder:SetBackdropColor(unpack(cfg.backdropcolor))
			self.TotemBarBorder:SetBackdropBorderColor(unpack(cfg.brdcolor))	
			self.TotemBarBorder:SetFrameLevel(0)	
		end
	end
end

-------------------------
-- unit specific stuff --
-------------------------
local UnitSpecific = {
	player = function(self, ...)
		Shared(self, ...)
		
		if cfg.useCastbar then	
			createCastbar(self)	

			-- disable blizzards pet castbar
			PetCastingBarFrame:UnregisterAllEvents()
			PetCastingBarFrame.Show = function() end
			PetCastingBarFrame:Hide()
			
			self.Castbar:SetAllPoints(self.Info)
			self.Castbar.Text:SetPoint("LEFT", self.Info, "LEFT", 4, 0)	
			self.Castbar.Time2:SetPoint("RIGHT", self.Castbar, "RIGHT", -2, 0)
			self.Castbar.Time:SetPoint("RIGHT", self.Castbar.Time2, "LEFT", 0, 0)

			if cfg.useSpellIcon then 
				if not cfg.PlayerRightSideSpellIcon then
					self.Castbar.Icon:SetPoint("TOPRIGHT", self, "TOPLEFT", -8, 0)
				else
					self.Castbar.Icon:SetPoint("TOPLEFT", self, "TOPRIGHT", 8, 0)
				end
			end
		end
		
		self.Health.value:SetPoint("RIGHT", self.Info, "RIGHT", -4, 0)	
		self.Power.value:SetPoint("RIGHT", self.Health.value, "LEFT", -2, 0)	
		
		if playerClass == "DRUID" then
			self.Name:SetPoint("CENTER", self.Info, 0, 1)
			self:Tag(self.Name, '[druidPower]')
		end
		
		self.Health:SetHeight(cfg.heightP)
		self.Health:SetWidth(cfg.widthP)	
		self.Power:SetWidth(cfg.widthP)		

		if cfg.usePortrait then		
			createPortraits(self)	
			if cfg.separatePortraitPlayer then
				self.Portrait:ClearAllPoints()
				self.Portrait:SetSize(cfg.widthP*0.4, cfg.heightP)
				
				if cfg.MirrorPlayerPortrait then
					self.Portrait:SetPoint("TOPRIGHT", self, "TOPRIGHT", 0, 0)
					self.Health:SetPoint("RIGHT", self, "RIGHT", -((cfg.widthP*0.4)+4), 0)
				else
					self.Health:SetPoint("LEFT", self, "LEFT", (cfg.widthP*0.4)+4, 0)
					self.Portrait:SetPoint("TOPLEFT", self, "TOPLEFT", 0, 0)
				end
				
				self.Health:SetWidth(cfg.widthP*0.6)
				createPortraitBGBorder(self)
			end	
			self.Portrait.PostUpdate = Portrait_PostUpdate	
		end

		-- buffs
		if cfg.HideBlizzardAuras then
			createBuffs(self)
			if cfg.PlayerBuffsOnPlayerFrame then
				if playerClass == "WARLOCK" or playerClass == "PALADIN" or playerClass == "DEATHKNIGHT" or playerClass == "DRUID" then
					self.Buffs:SetPoint("TOP", self, "BOTTOM", 0, -(12 + cfg.ClassBarHeight))
				else
					self.Buffs:SetPoint("TOP", self, "BOTTOM", 0, -10)
				end
				self.Buffs.initialAnchor = "TOPLEFT"	
				self.Buffs["growth-x"] = "RIGHT"		
				self.Buffs["growth-y"] = "DOWN"
				self.Buffs.size = ((cfg.widthP - ((cfg.PlayerBuffNumber-1)*4)) / cfg.PlayerBuffNumber)
				self.Buffs:SetHeight(self.Buffs.size*4)
			else
				self.Buffs:ClearAllPoints()
				self.Buffs:SetPoint(cfg.PlayerBufAnchor1, cfg.PlayerBufParent, cfg.PlayerBufAnchor2, cfg.PlayerBufOffset_X, cfg.PlayerBufOffset_Y)
				self.Buffs.initialAnchor = cfg.PlayerBuffInitialAnchor	
				self.Buffs["growth-x"] = cfg.PlayerBuffGrowth_X
				self.Buffs["growth-y"] = cfg.PlayerBuffGrowth_Y
				self.Buffs.size = cfg.PlayerBuffSize
				self.Buffs:SetWidth(self.Buffs.size*10)
				self.Buffs:SetHeight(self.Buffs.size*4)
				self.Buffs.spacing = 6
			end
		end
		
		-- debuffs
		createDebuffs(self)
		self.Debuffs:SetPoint("BOTTOM", self.Power, "TOP", 0, 8)
		self.Debuffs.initialAnchor = "BOTTOMRIGHT"
		self.Debuffs["growth-x"] = "LEFT"		
		self.Debuffs["growth-y"] = "UP"	
		self.Debuffs.size = ((cfg.widthP - ((cfg.PlayerDebuffNumber-1)*4)) / cfg.PlayerDebuffNumber)
		self.Debuffs:SetHeight(self.Debuffs.size*2)	
			
		-- Icons
		createRaidIcon(self)
		self.RaidIcon:SetPoint("LEFT", self.Health, "LEFT", 4, 0)
		
		RIc = self.Health:CreateTexture(nil, 'OVERLAY')
		RIc:SetSize(12, 12)	
		RIc:SetPoint("LEFT", self.Info, "LEFT", 4, 0)
		self.Resting = RIc
		
		CIc = self.Health:CreateTexture(nil, 'OVERLAY')
		CIc:SetSize(14, 14)			
		CIc:SetPoint("LEFT", RIc, "RIGHT", 4, 0)	
		self.Combat = CIc			
		
		LIc = self.Health:CreateTexture(nil, "OVERLAY")
		LIc:SetSize(12, 12)			
		LIc:SetPoint("LEFT", CIc, "RIGHT", 4, 0)
		self.Leader = LIc
		
		MLIc = self.Health:CreateTexture(nil, 'OVERLAY')
		MLIc:SetSize(12, 12)	
		MLIc:SetPoint("LEFT", LIc, "RIGHT", 4, 0)
		self.MasterLooter = MLIc	
		
		-- plugins
		CombatFeedback(self)
		SmoothUpdate(self)
		BarFader(self)
		
		-- TotemBar
		if playerClass == "SHAMAN" then		
			TotemBar(self)

			if cfg.PlayerBuffsOnPlayerFrame and IsAddOnLoaded("oUF_TotemBar") then
				self.Buffs:SetPoint("TOP", self.Info, "BOTTOM", 0, -(10 + cfg.ClassBarHeight))			
			end			
		end
		
		-- update debuff highlight
		self:RegisterEvent("UNIT_AURA", updateDispel)
		
		self.FrameGlow:SetPoint("TOPLEFT", self.Info, "TOPLEFT", -7, (7 + cfg.heightP + cfg.heightPP + cfg.PPyOffset + cfg.InfoOffset))
		self.FrameGlow:SetPoint("BOTTOMRIGHT", self.Info, "BOTTOMRIGHT", 7, -7)		
		
		self:SetSize(cfg.widthP, cfg.heightP + cfg.NumbFS + cfg.PPyOffset + cfg.InfoOffset)
	end,
	
	WpnEnch = function(self, ...)
		WeaponEnchant(self)
		BarFader(self)
	
		self:SetSize(cfg.WeapEnchantIconSize*2, cfg.WeapEnchantIconSize)
	end,
	
	pClassbar = function(self, ...)
		local playerClass = select(2, UnitClass("player"))	
		
		-- plugins		
		BarFader(self)
		
		if playerClass == "PALADIN" then
			self.HolyPower = CreateFrame("Frame", nil, self)
			for i = 1, 3 do
				self.HolyPower[i] = CreateFrame("StatusBar", self:GetName().."_Shards"..i, self)
				self.HolyPower[i]:SetHeight(cfg.ClassBarHeight)
				self.HolyPower[i]:SetWidth((cfg.widthCB - 6) / 3)
				if (i == 1) then
					self.HolyPower[i]:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, 0)
				else
					self.HolyPower[i]:SetPoint("TOPLEFT", self.HolyPower[i-1], "TOPRIGHT", 3, 0)
				end
				self.HolyPower[i]:SetStatusBarTexture(cfg.HPtex)
				local color = self.colors.power["HOLY_POWER"]
				self.HolyPower[i]:SetStatusBarColor(color[1], color[2], color[3])

				self.HolyPowerBorder = CreateFrame("Frame", nil, self)
				self.HolyPowerBorder:SetPoint("TOPLEFT", self.HolyPower[i], "TOPLEFT", -2, 2)
				self.HolyPowerBorder:SetPoint("BOTTOMRIGHT", self.HolyPower[i], "BOTTOMRIGHT", 2, -2)					
				self.HolyPowerBorder:SetBackdrop(backdrop)
				self.HolyPowerBorder:SetBackdropColor(unpack(cfg.backdropcolor))
				self.HolyPowerBorder:SetBackdropBorderColor(unpack(cfg.brdcolor))	
				self.HolyPowerBorder:SetFrameLevel(0)	
			end
		end
		
		if playerClass == "WARLOCK" then
			self.SoulShards = CreateFrame("Frame", nil, self)
			for i = 1, 3 do
				self.SoulShards[i] = CreateFrame("StatusBar", self:GetName().."_Shards"..i, self)
				self.SoulShards[i]:SetHeight(cfg.ClassBarHeight)
				self.SoulShards[i]:SetWidth((cfg.widthCB - 6) / 3)
				if (i == 1) then
					self.SoulShards[i]:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, 0)
				else
					self.SoulShards[i]:SetPoint("TOPLEFT", self.SoulShards[i-1], "TOPRIGHT", 3, 0)
				end
				self.SoulShards[i]:SetStatusBarTexture(cfg.HPtex)

				local color = self.colors.power["SOUL_SHARDS"]
				self.SoulShards[i]:SetStatusBarColor(color[1], color[2], color[3])

				self.SoulShardsBorder = CreateFrame("Frame", nil, self)
				self.SoulShardsBorder:SetPoint("TOPLEFT", self.SoulShards[i], "TOPLEFT", -2, 2)
				self.SoulShardsBorder:SetPoint("BOTTOMRIGHT", self.SoulShards[i], "BOTTOMRIGHT", 2, -2)					
				self.SoulShardsBorder:SetBackdrop(backdrop)
				self.SoulShardsBorder:SetBackdropColor(unpack(cfg.backdropcolor))
				self.SoulShardsBorder:SetBackdropBorderColor(unpack(cfg.brdcolor))	
				self.SoulShardsBorder:SetFrameLevel(0)	
			end
		end			
			
		if playerClass =="DEATHKNIGHT" then	
			self.Runes = CreateFrame("Frame", nil, self)
			for i = 1, 6 do
				self.Runes[i] = CreateFrame("StatusBar", self:GetName().."_Runes"..i, self)
				self.Runes[i]:SetHeight(cfg.ClassBarHeight)
				self.Runes[i]:SetWidth((cfg.widthCB - 15) / 6)
				if (i == 1) then
					self.Runes[i]:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, 0)
				else
					self.Runes[i]:SetPoint("TOPLEFT", self.Runes[i-1], "TOPRIGHT", 3, 0)
				end
				self.Runes[i]:SetStatusBarTexture(cfg.HPtex)

				self.Runes[i].bg = self.Runes[i]:CreateTexture(nil, "BORDER")
				self.Runes[i].bg:SetPoint("TOPLEFT", self.Runes[i], "TOPLEFT", -0, 0)
				self.Runes[i].bg:SetPoint("BOTTOMRIGHT", self.Runes[i], "BOTTOMRIGHT", 0, -0)				
				self.Runes[i].bg:SetTexture(cfg.Itex)
				self.Runes[i].bg.multiplier = 0.34
				
				self.RunesBorder = CreateFrame("Frame", nil, self)
				self.RunesBorder:SetPoint("TOPLEFT", self.Runes[i], "TOPLEFT", -2, 2)
				self.RunesBorder:SetPoint("BOTTOMRIGHT", self.Runes[i], "BOTTOMRIGHT", 2, -2)					
				self.RunesBorder:SetBackdrop(backdrop)
				self.RunesBorder:SetBackdropColor(unpack(cfg.backdropcolor))
				self.RunesBorder:SetBackdropBorderColor(unpack(cfg.brdcolor))	
				self.RunesBorder:SetFrameLevel(0)	
			end
		end		
		
		if playerClass == 'DRUID' then
			local eclipseBar = CreateFrame('Frame', nil, self)
			eclipseBar:SetPoint('TOPLEFT', self, 'BOTTOMLEFT', 0, 0)
			eclipseBar:SetSize(cfg.widthCB, cfg.ClassBarHeight)
			
			eclipseBar.Border = CreateFrame("Frame", nil, self)
			eclipseBar.Border:SetPoint("TOPLEFT", eclipseBar, "TOPLEFT", -2, 2)
			eclipseBar.Border:SetPoint("BOTTOMRIGHT", eclipseBar, "BOTTOMRIGHT", 2, -2)					
			eclipseBar.Border:SetBackdrop(backdrop)
			eclipseBar.Border:SetBackdropColor(unpack(cfg.backdropcolor))
			eclipseBar.Border:SetBackdropBorderColor(unpack(cfg.brdcolor))	
			eclipseBar.Border:SetFrameLevel(0)				

			local lunarBar = CreateFrame('StatusBar', nil, eclipseBar)
			lunarBar:SetPoint('LEFT', eclipseBar, 'LEFT', 0, 0)
			lunarBar:SetSize(cfg.widthCB, cfg.ClassBarHeight)
			lunarBar:SetStatusBarTexture(cfg.HPtex)
			lunarBar:SetStatusBarColor(0.34, 0.1, 0.86)
			eclipseBar.LunarBar = lunarBar
	
			local solarBar = CreateFrame('StatusBar', nil, eclipseBar)
			solarBar:SetPoint('LEFT', lunarBar:GetStatusBarTexture(), 'RIGHT', 0, 0)
			solarBar:SetSize(cfg.widthCB, cfg.ClassBarHeight)
			solarBar:SetStatusBarTexture(cfg.HPtex)
			solarBar:SetStatusBarColor(0.95, 0.73, 0.15)
			eclipseBar.SolarBar = solarBar
	
			local eclipseBarText = solarBar:CreateFontString(nil, 'OVERLAY')
			eclipseBarText:SetPoint('CENTER', eclipseBar, 'CENTER', 0, 0)
			eclipseBarText:SetFont(cfg.NumbFont, cfg.NumbFS, cfg.fontFNum)
			self:Tag(eclipseBarText, '[pereclipse]%')
		
			self.EclipseBar = eclipseBar
		end		

		self:SetSize(cfg.widthCB, cfg.heightCB)
	end,	
	
	target = function(self, ...)
		Shared(self, ...)
		self.Health:SetHeight(cfg.heightT)
		self.Health:SetWidth(cfg.widthT)	
		self.Power:SetWidth(cfg.widthT)	
		
		if cfg.useCastbar then	
			createCastbar(self)			
			self.Castbar:SetAllPoints(self.Info)
			self.Castbar.Text:SetPoint("LEFT", self.Info, "LEFT", 4, 0)	
			self.Castbar.Time2:SetPoint("RIGHT", self.Castbar, "RIGHT", -2, 0)
			self.Castbar.Time:SetPoint("RIGHT", self.Castbar.Time2, "LEFT", 0, 0)
			
			if cfg.useSpellIcon then 
				if not cfg.TargetRightSideSpellIcon then
					self.Castbar.Icon:SetPoint("TOPRIGHT", self, "TOPLEFT", -8, 0)
				else
					self.Castbar.Icon:SetPoint("TOPLEFT", self, "TOPRIGHT", 8, 0)
				end
			end
		end
		
		self.Health.value:SetPoint("RIGHT", self.Info, "RIGHT", -4, 0)	
		self.Power.value:SetPoint("RIGHT", self.Health.value, "LEFT", -2, 0)		
		self.Status:SetPoint("LEFT", self.Info, "LEFT", 2, 0)
		self.Name:SetPoint("LEFT", self.Status, "RIGHT", 2, 0)
		
		if cfg.usePortrait then		
			createPortraits(self)
			if cfg.separatePortraitTarget then
				self.Portrait:ClearAllPoints()
				self.Portrait:SetSize(cfg.widthT*0.4, cfg.heightT)
				
				if cfg.MirrorTargetPortrait then
					self.Portrait:SetPoint("TOPRIGHT", self, "TOPRIGHT", 0, 0)
					self.Health:SetPoint("RIGHT", self, "RIGHT", -((cfg.widthT*0.4)+4), 0)
				else
					self.Health:SetPoint("LEFT", self, "LEFT", (cfg.widthT*0.4)+4, 0)
					self.Portrait:SetPoint("TOPLEFT", self, "TOPLEFT", 0, 0)
				end				
				
				self.Health:SetWidth(cfg.widthT*0.6)
				createPortraitBGBorder(self)
			end				
			self.Portrait.PostUpdate = Portrait_PostUpdate		
		end
		
		-- combo point font string
		local cpoints = self.Health:CreateFontString(nil, 'OVERLAY')
		cpoints:SetPoint("RIGHT", self.Health, "RIGHT", -8, 0)
		cpoints:SetFont(cfg.NumbFont, cfg.ComboFS, cfg.fontFNum)
		cpoints:SetTextColor(unpack(cfg.sndcolor))
		self:Tag(cpoints, '[cpoints]')

		createBuffs(self)	
		self.Buffs:SetPoint("TOP", self, "BOTTOM", 0, -8)
		self.Buffs.initialAnchor = "TOPLEFT"	
		self.Buffs["growth-x"] = "RIGHT"		
		self.Buffs["growth-y"] = "DOWN"
		self.Buffs.size = ((cfg.widthT - ((cfg.BuffNumberTar-1)*4)) / cfg.BuffNumberTar)
		self.Buffs:SetHeight(self.Buffs.size*4)
		
		createDebuffs(self)
		self.Debuffs:SetPoint("BOTTOM", self.Power, "TOP", 0, 8)
		self.Debuffs.initialAnchor = "BOTTOMLEFT"
		self.Debuffs["growth-x"] = "RIGHT"		
		self.Debuffs["growth-y"] = "UP"	
		self.Debuffs.num = cfg.DebuffNumberTar*2
		self.Debuffs.size = ((cfg.widthT - ((cfg.DebuffNumberTar-1)*4)) / cfg.DebuffNumberTar)
		self.Debuffs:SetHeight(self.Debuffs.size*2)	
		
		if cfg.onlyShowPlayerBuffs then
			self.Buffs.onlyShowPlayer = true
		end	
		
		if cfg.onlyShowPlayerDebuffs then
			self.Debuffs.onlyShowPlayer = true
		end	
		
		-- plugins
		SpellRange(self)
		CombatFeedback(self)
		SmoothUpdate(self)
		
		-- Icons
		createRaidIcon(self)
		createQuestIcon(self)
		createPhaseIcon(self)		
		self.RaidIcon:SetPoint("LEFT", self.Health, "LEFT", 4, 0)		
		self.QuestIcon:SetPoint("LEFT", self.RaidIcon, "RIGHT", 4, 0)	
		self.PhaseIcon:SetPoint("LEFT", self.QuestIcon, "RIGHT", 4, 0)
		
		-- update debuff highlight
		self:RegisterEvent("UNIT_AURA", updateDispel)	

		self.FrameGlow:SetPoint("TOPLEFT", self.Info, "TOPLEFT", -7, (7 + cfg.heightT + cfg.heightPP + cfg.PPyOffset + cfg.InfoOffset))
		self.FrameGlow:SetPoint("BOTTOMRIGHT", self.Info, "BOTTOMRIGHT", 7, -7)	
		
		self:SetSize(cfg.widthT, cfg.heightT + cfg.NumbFS + cfg.PPyOffset + cfg.InfoOffset)
	end,
	
	focus = function(self, ...)
		Shared(self, ...)
		self.Health:SetHeight(cfg.heightF)
		self.Health:SetWidth(cfg.widthF)	
		self.Power:SetWidth(cfg.widthF)	
		
		if cfg.useCastbar then	
			createCastbar(self)			
			self.Castbar:SetAllPoints(self.Info)
			self.Castbar.Text:SetPoint("LEFT", self.Info, "LEFT", 4, 0)	
			self.Castbar.Time2:SetPoint("RIGHT", self.Castbar, "RIGHT", -2, 0)
			self.Castbar.Time:SetPoint("RIGHT", self.Castbar.Time2, "LEFT", 0, 0)
			
			if cfg.useSpellIcon then 
				if not cfg.FocusRightSideSpellIcon then
					self.Castbar.Icon:SetPoint("TOPRIGHT", self, "TOPLEFT", -8, 0)
				else
					self.Castbar.Icon:SetPoint("TOPLEFT", self, "TOPRIGHT", 8, 0)
				end
			end
		end
		
		self.Health.value:SetPoint("RIGHT", self.Info, "RIGHT", -4, 0)			
		self.Power.value:SetPoint("RIGHT", self.Health.value, "LEFT", -2, 0)		
		self.Status:SetPoint("LEFT", self.Info, "LEFT", 2, 0)
		self.Name:SetPoint("LEFT", self.Status, "RIGHT", 2, 0)
		
		if cfg.usePortrait then		
			createPortraits(self)
			if cfg.separatePortraitFocus then
				self.Portrait:ClearAllPoints()
				self.Portrait:SetSize(cfg.widthF*0.4, cfg.heightF)

				if cfg.MirrorFocusPortrait then
					self.Portrait:SetPoint("TOPRIGHT", self, "TOPRIGHT", 0, 0)
					self.Health:SetPoint("RIGHT", self, "RIGHT", -((cfg.widthF*0.4)+4), 0)
				else
					self.Health:SetPoint("LEFT", self, "LEFT", (cfg.widthF*0.4)+4, 0)
					self.Portrait:SetPoint("TOPLEFT", self, "TOPLEFT", 0, 0)
				end		
				
				self.Health:SetWidth(cfg.widthF*0.6)
				createPortraitBGBorder(self)
			end				
			self.Portrait.PostUpdate = Portrait_PostUpdate		
		end
		
		createBuffs(self)	
		self.Buffs:SetPoint("TOP", self, "BOTTOM", 0, -8)
		self.Buffs.initialAnchor = "TOPLEFT"	
		self.Buffs["growth-x"] = "RIGHT"		
		self.Buffs["growth-y"] = "DOWN"
		self.Buffs.size = ((cfg.widthF - ((cfg.BuffNumberFoc-1)*4)) / cfg.BuffNumberFoc)
		self.Buffs:SetHeight(self.Buffs.size*4)		
		
		createDebuffs(self)
		self.Debuffs:SetPoint("BOTTOM", self.Power, "TOP", 0, 8)
		self.Debuffs.initialAnchor = "BOTTOMLEFT"
		self.Debuffs["growth-x"] = "RIGHT"		
		self.Debuffs["growth-y"] = "UP"		
		self.Debuffs.num = cfg.DebuffNumberFoc*2		
		self.Debuffs.size = ((cfg.widthF - ((cfg.DebuffNumberFoc-1)*4)) / cfg.DebuffNumberFoc)
		self.Debuffs:SetHeight(self.Debuffs.size*2)			
		
		if cfg.onlyShowPlayerBuffs then
			self.Buffs.onlyShowPlayer = true
		end	
		
		if cfg.onlyShowPlayerDebuffs then
			self.Debuffs.onlyShowPlayer = true
		end	
		
		-- plugins
		SpellRange(self)
		CombatFeedback(self)
		SmoothUpdate(self)
		
		-- Icons
		createRaidIcon(self)
		createQuestIcon(self)
		self.RaidIcon:SetPoint("LEFT", self.Health, "LEFT", 4, 0)		
		self.QuestIcon:SetPoint("LEFT", self.RaidIcon, "RIGHT", 4, 0)		
	
		-- update and debuff highlight
		self:RegisterEvent("UNIT_AURA", updateDispel)
		
		self.FrameGlow:SetPoint("TOPLEFT", self.Info, "TOPLEFT", -7, (7 + cfg.heightF + cfg.heightPP + cfg.PPyOffset + cfg.InfoOffset))
		self.FrameGlow:SetPoint("BOTTOMRIGHT", self.Info, "BOTTOMRIGHT", 7, -7)	
		
		self:SetSize(cfg.widthF, cfg.heightF + cfg.NumbFS + cfg.PPyOffset + cfg.InfoOffset)
	end,	

	pet = function(self, ...)
		Shared(self, ...)
		
		self.Health:SetHeight(cfg.heightS)
		self.Health:SetWidth(cfg.widthS)			
		self.Power:SetWidth(cfg.widthS)
		self.Name:SetPoint("CENTER", self.Info, 0, 0)
		self:Tag(self.Name, '[raidcolor][shortname]')		
		
		self.Power.colorPower = true			
		
		-- plugins
		SpellRange(self)
		BarFader(self)		

		-- Icons
		createRaidIcon(self)
		self.RaidIcon:SetPoint("CENTER", self.Health, "CENTER", 0, 0)	
		
		-- update and debuff highlight
		self:RegisterEvent("UNIT_AURA", updateDispel)
		
		self.FrameGlow:SetPoint("TOPLEFT", self.Info, "TOPLEFT", -7, (7 + cfg.heightS + cfg.heightPP + cfg.PPyOffset + cfg.InfoOffset))
		self.FrameGlow:SetPoint("BOTTOMRIGHT", self.Info, "BOTTOMRIGHT", 7, -7)	
		
		self:SetSize(cfg.widthS, cfg.heightS + cfg.NumbFS + cfg.PPyOffset + cfg.InfoOffset)
	end,
	
	targettarget = function(self, ...)
		Shared(self, ...)	

		self.Health:SetHeight(cfg.heightS)
		self.Health:SetWidth(cfg.widthS)			
		self.Power:SetWidth(cfg.widthS)	
		self.Name:SetPoint("CENTER", self.Info, 0, 0)
		self:Tag(self.Name, '[raidcolor][shortname]')
		
		-- plugins
		SpellRange(self)
		
		-- Icons
		createRaidIcon(self)
		self.RaidIcon:SetPoint("CENTER", self.Health, "CENTER", 0, 0)		
		
		-- update and debuff highlight
		self:RegisterEvent("UNIT_AURA", updateDispel)		
		
		self.FrameGlow:SetPoint("TOPLEFT", self.Info, "TOPLEFT", -7, (7 + cfg.heightS + cfg.heightPP + cfg.PPyOffset + cfg.InfoOffset))
		self.FrameGlow:SetPoint("BOTTOMRIGHT", self.Info, "BOTTOMRIGHT", 7, -7)	
		
		self:SetSize(cfg.widthS, cfg.heightS + cfg.NumbFS + cfg.PPyOffset + cfg.InfoOffset)
	end,

	focustarget = function(self, ...)
		Shared(self, ...)
		
		self.Health:SetHeight(cfg.heightS)
		self.Health:SetWidth(cfg.widthS)			
		self.Power:SetWidth(cfg.widthS)		
		self.Name:SetPoint("CENTER", self.Info, 0, 0)	
		self:Tag(self.Name, '[raidcolor][shortname]')
		
		-- plugins
		SpellRange(self)
		
		-- Icons
		createRaidIcon(self)
		self.RaidIcon:SetPoint("CENTER", self.Health, "CENTER", 0, 0)	
		
		-- update and debuff highlight
		self:RegisterEvent("UNIT_AURA", updateDispel)		
		
		self.FrameGlow:SetPoint("TOPLEFT", self.Info, "TOPLEFT", -7, (7 + cfg.heightS + cfg.heightPP + cfg.PPyOffset + cfg.InfoOffset))
		self.FrameGlow:SetPoint("BOTTOMRIGHT", self.Info, "BOTTOMRIGHT", 7, -7)	
		
		self:SetSize(cfg.widthS, cfg.heightS + cfg.NumbFS + cfg.PPyOffset + cfg.InfoOffset)
	end,

	boss = function(self, ...)
		Shared(self, ...)
		
		self.Health:SetHeight(cfg.heightM)
		self.Health:SetWidth(cfg.widthM)				
		self.Power:SetWidth(cfg.widthM)	
		self.Name:SetPoint("LEFT", self.Health, "LEFT", 4, 0)	
		self.Health.value:SetPoint("RIGHT", self.Health, "RIGHT", -4, 0)	
		
		self:Tag(self.Name, '[afkdnd][raidcolor][abbrevname]')		
		
		-- kill info panel
		self.Info:ClearAllPoints()
		self.InfoBG:ClearAllPoints()
		
		-- plugins
		SpellRange(self)
		
		-- Icons
		createRaidIcon(self)
		self.RaidIcon:SetPoint("CENTER", self.Health, "CENTER", 0, 0)		
		
		self.FrameGlow:SetPoint("TOPLEFT", self.Health, "TOPLEFT", -7, (7 + cfg.heightM + cfg.heightPP + cfg.PPyOffset + cfg.InfoOffset))
		self.FrameGlow:SetPoint("BOTTOMRIGHT", self.Health, "BOTTOMRIGHT", 7, -7)	
		
		self:SetSize(cfg.widthM, cfg.heightM + cfg.NumbFS + cfg.PPyOffset)
	end,
	
	MainTank = function(self, ...)
		Shared(self, ...)
		
		self.Health:SetHeight(cfg.heightM)
		self.Health:SetWidth(cfg.widthM)				
		self.Power:Hide()
		self.Name:SetPoint("LEFT", self.Health, "LEFT", 4, 0)
		self.Health.value:SetPoint("RIGHT", self.Health, "RIGHT", -4, 0)		

		self:Tag(self.Name, '[afkdnd][raidcolor][abbrevname]')	
		
		-- kill info panel
		self.Info:ClearAllPoints()
		self.InfoBG:ClearAllPoints()

		-- plugins
		SpellRange(self)
		
		-- Icons
		createRaidIcon(self)
		self.RaidIcon:SetPoint("CENTER", self.Health, "CENTER", 0, 0)

		self.FrameGlow:SetPoint("TOPLEFT", self.Health, "TOPLEFT", -7, (7 + cfg.heightM + cfg.heightPP + cfg.PPyOffset + cfg.InfoOffset))
		self.FrameGlow:SetPoint("BOTTOMRIGHT", self.Health, "BOTTOMRIGHT", 7, -7)	
		
		self:SetSize(cfg.widthM, cfg.heightM + cfg.NumbFS + cfg.PPyOffset)		
	end,
	
	arenaframes = function(self, ...)
		Shared(self, ...)
		self.Health:SetHeight(cfg.heightPA)
		self.Health:SetWidth(cfg.widthPA)	
		self.Power:SetWidth(cfg.widthPA)

		self.Health.value:SetPoint("RIGHT", self.Info, "RIGHT", -4, 0)			
		self.Power.value:SetPoint("RIGHT", self.Health.value, "LEFT", -2, 0)		
		self.Status:SetPoint("LEFT", self.Info, "LEFT", 2, 0)
		self.Name:SetPoint("LEFT", self.Status, "RIGHT", 2, 0)
		
		if cfg.usePartyPortrait then
			createPortraits(self)
			if cfg.separatePortraitParty then
				self.Portrait:ClearAllPoints()
				self.Portrait:SetSize(cfg.widthPA*0.4, cfg.heightPA)
				self.Portrait:SetPoint("TOPLEFT", self, "TOPLEFT", 0, 0)
				
				self.Health:SetPoint("LEFT", (cfg.widthPA*0.4)+4, 0)
				self.Health:SetWidth(cfg.widthPA*0.6)
				createPortraitBGBorder(self)
			end				
			self.Portrait.PostUpdate = Portrait_PostUpdate
		end
		
		local Auras = CreateFrame("Frame", nil, self)
		Auras:SetHeight(cfg.heightPA)
		Auras:SetPoint("TOPRIGHT", self, "TOPLEFT", -9, 9)
		Auras.initialAnchor = "TOPRIGHT"
		Auras:SetWidth((cfg.heightPA) * 12)
		Auras.size = (cfg.heightPA+8)
		Auras.gap = true
		Auras.numBuffs = 8
		Auras.numDebuffs = 4
		Auras.spacing = 4
		Auras["growth-x"] = "LEFT"

		Auras.PostCreateIcon = PostCreateIcon
		Auras.PostUpdateIcon = PostUpdateIcon
		self.Auras = Auras	

		-- apply aura filter
		if cfg.FilterAuras then
			self.Auras.CustomFilter = CustomFilter
		end
		
		-- plugins
		CombatFeedback(self)
		SmoothUpdate(self)
		SpellRange(self)
		
		-- Icons
		createRaidIcon(self)
		self.RaidIcon:SetPoint("LEFT", self.Health, "LEFT", 4, 0)
		
		self.FrameGlow:SetPoint("TOPLEFT", self.Info, "TOPLEFT", -7, (7 + cfg.heightPA  + cfg.heightPP + cfg.PPyOffset + cfg.InfoOffset))
		self.FrameGlow:SetPoint("BOTTOMRIGHT", self.Info, "BOTTOMRIGHT", 7, -7)	
		
		createCastbar(self)
		self.Castbar:SetAllPoints(self.Info)
		self.Castbar.Text:SetPoint("LEFT", self.Info, "LEFT", 4, 0)	
		self.Castbar.Time2:SetPoint("RIGHT", self.Castbar, "RIGHT", -2, 0)
		self.Castbar.Time:SetPoint("RIGHT", self.Castbar.Time2, "LEFT", 0, 0)
		self.Castbar.Icon:SetSize(cfg.NumbFS+4,cfg.NumbFS+4)
		self.Castbar.Icon:SetPoint("BOTTOMRIGHT", self.Info, "BOTTOMLEFT", -10, 0)
			
		self:SetSize(cfg.widthPA, cfg.heightPA + cfg.NumbFS + cfg.PPyOffset + cfg.InfoOffset)
	end,	
		
	arenatargets = function(self, ...)
		Shared(self, ...)
			
		self.Health.value:ClearAllPoints()	
		self.Power.value:ClearAllPoints()		
		
		self.Health:SetHeight(cfg.heightPA)
		self.Health:SetWidth(cfg.widthS)			
		self.Power:SetWidth(cfg.widthS)		
		self.Name:SetPoint("CENTER", self.Info, 0, 0)	
		self:Tag(self.Name, '[raidcolor][shortname]')
		
		-- Icons
		createRaidIcon(self)
		self.RaidIcon:SetPoint("CENTER", self.Health, "CENTER", 0, 0)	
		
		self.FrameGlow:SetPoint("TOPLEFT", self.Info, "TOPLEFT", -7, (7 + cfg.heightPA  + cfg.heightPP + cfg.PPyOffset + cfg.InfoOffset))
		self.FrameGlow:SetPoint("BOTTOMRIGHT", self.Info, "BOTTOMRIGHT", 7, -7)	
		
		self:SetSize(cfg.widthS, cfg.heightPA + cfg.NumbFS + cfg.PPyOffset + cfg.InfoOffset)
	end,			
}

-- raid, party
do
	local range = {
		insideAlpha = 1,
		outsideAlpha = cfg.FadeOutAlpha,
	}
	
	UnitSpecific.party = function(self, ...)
		Shared(self, ...)
		self.Health:SetHeight(cfg.heightPA)
		self.Health:SetWidth(cfg.widthPA)	
		self.Power:SetWidth(cfg.widthPA)
		
		self.Health.value:SetPoint("RIGHT", self.Info, "RIGHT", -4, 0)			
		self.Power.value:SetPoint("RIGHT", self.Health.value, "LEFT", -2, 0)		
		self.Status:SetPoint("LEFT", self.Info, "LEFT", 2, 0)
		self.Name:SetPoint("LEFT", self.Status, "RIGHT", 2, 0)
		
		if cfg.usePartyPortrait then
			createPortraits(self)
			if cfg.separatePortraitParty then
				self.Portrait:ClearAllPoints()
				self.Portrait:SetSize(cfg.widthPA*0.4, cfg.heightPA)
				self.Portrait:SetPoint("TOPLEFT", self, "TOPLEFT", 0, 0)
				
				self.Health:SetPoint("LEFT", (cfg.widthPA*0.4)+4, 0)
				self.Health:SetWidth(cfg.widthPA*0.6)
				createPortraitBGBorder(self)
			end				
			self.Portrait.PostUpdate = Portrait_PostUpdate
		end
		
		local Auras = CreateFrame("Frame", nil, self)
		Auras:SetHeight(cfg.heightPA)
		Auras:SetPoint("TOPLEFT", self, "TOPRIGHT", 10, 2)
		Auras.initialAnchor = "TOPLEFT"
		Auras:SetWidth((cfg.heightPA) * 12)
		Auras.size = (cfg.heightPA+10)
		Auras.gap = true
		Auras.numBuffs = 8
		Auras.numDebuffs = 4
		Auras.spacing = 4

		Auras.PostCreateIcon = PostCreateIcon
		Auras.PostUpdateIcon = PostUpdateIcon
		self.Auras = Auras	

		-- apply aura filter
		if cfg.FilterAuras then
			self.Auras.CustomFilter = CustomFilter
		end
		
		-- plugins
		HealComm4(self)
		CombatFeedback(self)
		SmoothUpdate(self)
		self.Range = range
		
		-- Icons
		createRaidIcon(self)
		createPhaseIcon(self)		
		self.RaidIcon:SetPoint("LEFT", self.Health, "LEFT", 4, 0)
		self.PhaseIcon:SetPoint("LEFT", self.RaidIcon, "RIGHT", 4, 0)
	
		LfDR = self.Health:CreateTexture(nil, 'OVERLAY')
		LfDR:SetSize(12, 12)
		LfDR:SetPoint("LEFT", self.Name, "RIGHT", 6, 0)		
		self.LFDRole = LfDR
		
		LIc = self.Health:CreateTexture(nil, "OVERLAY")
		LIc:SetSize(12, 12)			
		LIc:SetPoint("LEFT", LfDR, "RIGHT", 4, 0)
		self.Leader = LIc
		
		MLIc = self.Health:CreateTexture(nil, 'OVERLAY')
		MLIc:SetSize(12, 12)	
		MLIc:SetPoint("LEFT", LIc, "RIGHT", 4, 0)
		self.MasterLooter = MLIc
		
		rChk = self.Health:CreateTexture(nil, 'OVERLAY')
		rChk:SetSize(18, 18)
		rChk:SetPoint("CENTER", self.Health, 0, 0)		
		rChk.fadeTimer = 6
		self.ReadyCheck = rChk		
		
		self.FrameGlow:SetPoint("TOPLEFT", self.Info, "TOPLEFT", -7, (7 + cfg.heightPA  + cfg.heightPP + cfg.PPyOffset + cfg.InfoOffset))
		self.FrameGlow:SetPoint("BOTTOMRIGHT", self.Info, "BOTTOMRIGHT", 7, -7)	
				
		-- update and debuff highlight
		self:RegisterEvent("UNIT_AURA", updateDispel)
		
			-- party pets
			if (self:GetAttribute("unitsuffix") == "pet") then
				
				-- clear up the inherited mess ...
				self.Auras:ClearAllPoints()
				self.Name:ClearAllPoints()
				if cfg.usePartyPortrait then
					self.Portrait:ClearAllPoints()
					self.Health:SetPoint("LEFT")
				end	
				self.Health.value:ClearAllPoints()	
				self.Power.value:ClearAllPoints()		
		
				self.Health:SetWidth(cfg.widthS)			
				self.Power:SetWidth(cfg.widthS)		
				self.Name:SetPoint("CENTER", self.Info, 0, 0)	
				self:Tag(self.Name, '[raidcolor][shortname]')
		
				-- Icons
				self.RaidIcon:ClearAllPoints()
				self.RaidIcon:SetPoint("CENTER", self.Health, "CENTER", 0, 0)	
		
				self.FrameGlow:SetPoint("TOPLEFT", self.Info, "TOPLEFT", -7, (7 + cfg.heightPA  + cfg.heightPP + cfg.PPyOffset + cfg.InfoOffset))
				self.FrameGlow:SetPoint("BOTTOMRIGHT", self.Info, "BOTTOMRIGHT", 7, -7)	
		
				self:SetSize(cfg.widthS, cfg.heightPA + cfg.NumbFS + cfg.PPyOffset + cfg.InfoOffset)
			end
	end
	
	UnitSpecific.raid = function(self, ...)
		Shared(self, ...)
		
		
		self.Name:SetPoint("CENTER", self.Info, "CENTER", 0, 0)
		self:Tag(self.Name, '[raidcolor][raidhpname]')		

		self.Health:SetHeight(cfg.heightR)
		self.Health:SetWidth(cfg.widthR)	
		self.Power:SetWidth(cfg.widthR)			
		--self.Power:Hide()
		self.Power.PostUpdate = PostUpdatePowerRaid

		createDebuffs(self)		
		self.Debuffs:ClearAllPoints()
		self.Debuffs.size = 18
		self.Debuffs:SetWidth(self.Debuffs.size*2)
		self.Debuffs:SetHeight(self.Debuffs.size)		
		self.Debuffs:SetPoint("LEFT", self.Health, "LEFT", 0, 0)
		self.Debuffs.initialAnchor = "LEFT"
		self.Debuffs["growth-x"] = "RIGHT"	
		self.Debuffs.num = 2
		
		if cfg.FilterAuras then
			self.Debuffs.CustomFilter = CustomFilter
		end		
		
		-- class specific tags
		classTags(self)		

		-- plugins
		HealComm4(self)
		self.Range = range
		
		-- Icons
		createRaidIcon(self)
		self.RaidIcon:SetPoint("CENTER", self.Health, "CENTER", 0, 0)	
		
		rChk = self.Health:CreateTexture(nil, 'OVERLAY')
		rChk:SetSize(18, 18)
		rChk:SetPoint("CENTER", self.Health, 0, 0)		
		rChk.fadeTimer = 6
		self.ReadyCheck = rChk
		
		self.FrameGlow:SetPoint("TOPLEFT", self.Info, "TOPLEFT", -7, (7 + cfg.heightR  + cfg.heightPP + cfg.PPyOffset + cfg.InfoOffset))
		self.FrameGlow:SetPoint("BOTTOMRIGHT", self.Info, "BOTTOMRIGHT", 7, -7)	
				
		-- update and debuff highlight
		self:RegisterEvent("UNIT_AURA", updateDispel)
	end
end	 
 
---------------------------------------
-- register style(s) and spawn units --
---------------------------------------
 oUF:RegisterStyle("Nivea", Shared)

for unit,layout in next, UnitSpecific do
	oUF:RegisterStyle('Nivea - ' .. unit:gsub("^%l", string.upper), layout)
end

local spawnHelper = function(self, unit, ...)
	if(UnitSpecific[unit]) then
		self:SetActiveStyle('Nivea - ' .. unit:gsub("^%l", string.upper))
		local object = self:Spawn(unit)
		object:SetPoint(...)
		return object
	else
		self:SetActiveStyle'Nivea'
		local object = self:Spawn(unit)
		object:SetPoint(...)
		return object
	end
end

if cfg.ArenaFrames then
oUF:RegisterStyle('oUF_Nivea_Arena', UnitSpecific.arenaframes)
oUF:SetActiveStyle('oUF_Nivea_Arena')
local arena = {}
local arenatarget = {}
	for i = 1, 5 do
		arena[i] = oUF:Spawn("arena"..i, "oUF_Arena"..i)
		if i == 1 then
			arena[i]:SetPoint('RIGHT', UIParent, 'RIGHT', -100, 0)
		else
			arena[i]:SetPoint("BOTTOMRIGHT", arena[i-1], "TOPRIGHT", 0, 20)
		end
	end
	
oUF:RegisterStyle("oUF_Nivea_ArenaTarget", UnitSpecific.arenatargets)
oUF:SetActiveStyle("oUF_Nivea_ArenaTarget")
	for i = 1, 5 do
		arenatarget[i] = oUF:Spawn("arena"..i.."target", "oUF_Arena"..i.."target"):SetPoint("TOPLEFT",arena[i], "TOPRIGHT", 10, 0)
	end
end


oUF:Factory(function(self)
	
	local player = spawnHelper(self, 'player', "CENTER", -268, -218)
	spawnHelper(self, 'pet', "RIGHT", player, "LEFT", -10, 0)
	local target = spawnHelper(self, 'target', "CENTER", 268, -218)
	spawnHelper(self, 'targettarget', "LEFT", target, "RIGHT", 10, 0)
	local focus = spawnHelper(self, 'focus', "TOPLEFT", target, "BOTTOMLEFT", 0, -120)
	spawnHelper(self, 'focustarget', "LEFT", focus, "RIGHT", 10, 0)
	
if cfg.PartyFrames then
	self:SetActiveStyle'Nivea - Party'
	local party = self:SpawnHeader('oUF_Party', nil, 'custom  [group:party,nogroup:raid][@raid6,noexists,group:raid] show;hide', 	
		'showParty', true, 
		--'showPlayer', true, 
		'yOffset', -20, 
		'template', 'oUF_NiveaRaid',	-- party pets
		'oUF-initialConfigFunction', ([[
			self:SetWidth(%d)
			self:SetHeight(%d)
    ]]):format(cfg.widthPA, cfg.heightPA + cfg.NumbFS + cfg.PPyOffset + cfg.InfoOffset))
	party:SetPoint("BOTTOMLEFT", player, "TOPLEFT", 0, 60)
end

	
if cfg.RaidFrames then	
CompactRaidFrameManager:UnregisterAllEvents()
CompactRaidFrameManager:Hide()
CompactRaidFrameContainer:UnregisterAllEvents()
CompactRaidFrameContainer:Hide()	
	
	self:SetActiveStyle"Nivea - Raid"
	local raid = self:SpawnHeader('oUF_Raid', nil, 'raid10',		
	--local raid = self:SpawnHeader('oUF_Raid', nil, 'raid,party,solo',	
	--'showSolo', true,		
	'showRaid', true, 
	'showPlayer', true,
	'groupFilter', '1,2,3,4,5,6,7,8',
	'groupingOrder', '1,2,3,4,5,6,7,8',
	'groupBy', 'GROUP',
	'maxColumns', 8,
	'unitsPerColumn', 5,
	'column', 4,
	'columnAnchorPoint', "RIGHT",
	'columnSpacing', 8,
	'yOffset', -12,
	'oUF-initialConfigFunction', ([[
			self:SetWidth(%d)
			self:SetHeight(%d)
    ]]):format(cfg.widthR, cfg.heightR + cfg.NumbFS + cfg.PPyOffset + cfg.InfoOffset))
	--raid:SetPoint("BOTTOM", UIParent, "BOTTOM", 0, 150)	
	raid:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 30, -30)	
end

if cfg.BossFrames then		
	self:SetActiveStyle"Nivea - Boss"
	local boss = {}
		for i = 1, MAX_BOSS_FRAMES do
			local unit = self:Spawn("boss"..i, "oUF_NiveaBoss"..i)

			if i==1 then
				unit:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT", -80, -420)
			else
				unit:SetPoint("TOPLEFT", boss[i-1], "BOTTOMLEFT", 0, -10)
			end
			boss[i] = unit
		end
end
		
if cfg.MTFrames then		
	self:SetActiveStyle"Nivea - MainTank"
	local Main_Tank = self:SpawnHeader("oUF_MainTank", nil, 'raid, party, solo', 
		'showRaid', true, 
		"groupFilter", "MAINTANK", 
		'yOffset', -8, 
		"template", "oUF_NiveaMTartemplate",		-- MT Target
		'oUF-initialConfigFunction', ([[
			self:SetWidth(%d)
			self:SetHeight(%d)
		]]):format(cfg.widthM, cfg.heightM))		
	Main_Tank:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT", -80, -250)		
end
	
if cfg.useClassBar and playerClass == "WARLOCK" or playerClass == "PALADIN" or playerClass == "DEATHKNIGHT" or playerClass == "DRUID" then	
	oUF:RegisterStyle("oUF_Nivea_pClassbar", UnitSpecific.pClassbar)
	oUF:SetActiveStyle("oUF_Nivea_pClassbar")
	oUF:Spawn('player', 'pClassbar'):SetPoint("TOPLEFT", player, "BOTTOMLEFT", 0, 4)
end
	
if IsAddOnLoaded("oUF_WeaponEnchant") then	
	oUF:RegisterStyle("oUF_Nivea_WpnEnch", UnitSpecific.WpnEnch)
	oUF:SetActiveStyle("oUF_Nivea_WpnEnch")
	oUF:Spawn('player', 'WpnEnch'):SetPoint("BOTTOMRIGHT", player, "BOTTOMLEFT", -8, -(cfg.WeapEnchantIconSize+8))	
end

end)

-- remove SET_FOCUS & CLEAR_FOCUS from menu, to prevent errors
do 
    for k,v in pairs(UnitPopupMenus) do
        for x,y in pairs(UnitPopupMenus[k]) do
            if y == "SET_FOCUS" then
                table.remove(UnitPopupMenus[k],x)
            elseif y == "CLEAR_FOCUS" then
                table.remove(UnitPopupMenus[k],x)
            end
        end
    end
end

-------------
-- The End --
-------------