﻿local addon, ns = ...
local cfg = ns.cfg
local tags = CreateFrame("Frame") 

local utf8sub = function(string, i, dots)
	local bytes = string:len()
	if (bytes <= i) then
		return string
	else
		local len, pos = 0, 1
		while(pos <= bytes) do
			len = len + 1
			local c = string:byte(pos)
			if c > 240 then
				pos = pos + 4
			elseif c > 225 then
				pos = pos + 3
			elseif c > 192 then
				pos = pos + 2
			else
				pos = pos + 1
			end
			if (len == i) then break end
		end

		if (len == i and pos <= bytes) then
			return string:sub(1, pos - 1)..(dots and "..." or "")
		else
			return string
		end
	end
end

-----------------
-- custom tags --
-----------------

oUF.Tags["afkdnd"] = function(unit)
	if unit then
		return UnitIsAFK(unit) and "|cffffffff<AFK>|r " or UnitIsDND(unit) and "|cffffffff<DND>|r "
	end
end
oUF.TagEvents["afkdnd"] = "PLAYER_FLAGS_CHANGED"

oUF.Tags["abbrevname"] = function(unit)
	local oldName = UnitName(unit)
	local newName = (string.len(oldName) > 14) and string.gsub(oldName, "%s?(.[\128-\191]*)%S+%s", "%1. ") or oldName
	
	if not UnitIsPlayer(unit) then
		return utf8sub(newName, 14, false)
	else
		return oldName
	end
end
oUF.TagEvents["abbrevname"] = "UNIT_NAME_UPDATE"

if (not oUF.Tags["shortname"]) then
	oUF.Tags["shortname"] = function(unit)
		local oldName = UnitName(unit)
		local newName = (string.len(oldName) > 6) and string.gsub(oldName, "%s?(.[\128-\191]*)%S+%s", "%1. ") or oldName
		return utf8sub(newName, 6, false)
	end
end
oUF.TagEvents["shortname"] = "UNIT_NAME_UPDATE"

oUF.Tags["raidhpname"] = function(u)
local name = UnitName(u)
local c, m = UnitHealth(u), UnitHealthMax(u) 
if c == 0 then return "RIP" elseif(UnitIsGhost(u)) then return "GHO" elseif(not UnitIsConnected(u)) then return "OFF" elseif c < m then return oUF.Tags["missinghp"](u) else return utf8sub(name, 6, false) end end
oUF.TagEvents["raidhpname"] = "UNIT_HEALTH UNIT_MAXHEALTH UNIT_NAME_UPDATE"

oUF.Tags["druidPower"] = function(unit)
	local min, max = UnitPower(unit, 0), UnitPowerMax(unit, 0)
	return UnitPowerType(unit) ~= 0 and format('|cff0ff0ff %d|r', min)
end
oUF.TagEvents["druidPower"] = 'UNIT_MAXPOWER UNIT_POWER UPDATE_SHAPESHIFT_FORM'

---------------
-- raid tags --
---------------
oUF.Tags["NiveaPWS"] = function(u) 
	if UnitAura(u, GetSpellInfo(17) or "Power Word: Shield") then return "|cffFFD800S|r" end end
oUF.Tags["NiveaRenew"] = function(u) 
	local name, _,_,_,_,_,_, fromwho,_ = UnitAura(u, GetSpellInfo(139) or "Renew")
	if not (fromwho == "player") then return end
	if UnitAura(u, GetSpellInfo(139) or "Renew") then return "|cff00FF10R|r" end end
oUF.Tags["NiveaPoM"] = function(u) local c = select(4, UnitAura(u, GetSpellInfo(33076) or "Prayer of Mending")) if c then return "|cffFFCF7FP|r" end end
oUF.Tags["NiveaGS"] = function(u) if UnitAura(u, GetSpellInfo(47788) or "Guardian Spirit") then return "|cffFFD800G|r" end end
oUF.Tags["NiveaWS"] = function(u) if UnitDebuff(u, GetSpellInfo(6788) or "Weakened Soul") then return "|cffFF0000W|r" end end
oUF.Tags["NiveaFW"] = function(u) if UnitAura(u, GetSpellInfo(6346) or "Fear Ward") then return "|cffFF00FFF|r" end end

oUF.TagEvents["NiveaPWS"] = "UNIT_AURA"
oUF.TagEvents["NiveaPWB"] = "UNIT_AURA"
oUF.TagEvents["NiveaRenew"] = "UNIT_AURA"
oUF.TagEvents["NiveaPoM"] = "UNIT_AURA"
oUF.TagEvents["NiveaGS"] = "UNIT_AURA"
oUF.TagEvents["NiveaWS"] = "UNIT_AURA"
oUF.TagEvents["NiveaFW"] = "UNIT_AURA"

-- paladin
oUF.Tags["NiveaBoL"] = function(u) if UnitAura(u, GetSpellInfo(53563) or "Beacon of Light") then return "|cffFFD800B|r" end end

oUF.TagEvents["NiveaBoL"] = "UNIT_AURA"

-- druid
oUF.Tags["NiveaRej"] = function(u) 
	local name, _,_,_,_,_,_, fromwho,_ = UnitAura(u, GetSpellInfo(774) or "Rejuvenation")
	if not (fromwho == "player") then return end
	if UnitAura(u, GetSpellInfo(774) or "Rejuvenation") then return "|cff00FEBFR|r" end end
oUF.Tags["NiveaReg"] = function(u) 
	local name, _,_,_,_,_,_, fromwho,_ = UnitAura(u, GetSpellInfo(8936) or "Regrowth")
	if not (fromwho == "player") then return end
	if UnitAura(u, GetSpellInfo(8936) or "Regrowth") then return "|cff00FF10R|r" end end
oUF.Tags["NiveaLB"] = function(u) 
	local name, _,_,_,_,_,_, fromwho,_ = UnitAura(u, GetSpellInfo(33763) or "Lifebloom")
	if not (fromwho == "player") then return end
	local c = select(4, UnitAura(u, GetSpellInfo(33763) or "Lifebloom")) if c then return "|cffA7FD0AL|r" end end
oUF.Tags["NiveaInn"] = function(u) if UnitAura(u, GetSpellInfo(29166) or "Innervate") then return "|cff3c3be7I|r" end end
oUF.Tags["NiveaWG"] = function(u) if UnitAura(u, GetSpellInfo(53251) or "Wild Growth") then return "|cff33FF33W|r" end end

oUF.TagEvents["NiveaRej"] = "UNIT_AURA"
oUF.TagEvents["NiveaReg"] = "UNIT_AURA"
oUF.TagEvents["NiveaLB"] = "UNIT_AURA"
oUF.TagEvents["NiveaInn"] = "UNIT_AURA"
oUF.TagEvents["NiveaWG"] = "UNIT_AURA"

-- shaman
oUF.Tags["NiveaES"] = function(u) local c = select(4, UnitAura(u, GetSpellInfo(974) or "Earth Shield")) if c then return "|cffFFCF7FE|r" end end
oUF.Tags["NiveaRipT"] = function(u) 
	local name, _,_,_,_,_,_, fromwho,_ = UnitAura(u, GetSpellInfo(61295) or "Riptide")
	if not (fromwho == "player") then return end
	if UnitAura(u, GetSpellInfo(61295) or "Riptide") then return "|cff00FEBFR|r" end end
oUF.Tags["NiveaErLiv"] = function(u) 
	local name, _,_,_,_,_,_, fromwho,_ = UnitAura(u, GetSpellInfo(52000) or "Earthliving")
	if not (fromwho == "player") then return end
	if UnitAura(u, GetSpellInfo(51945) or "Earthliving") then return "|cff00FEBFL|r" end end

oUF.TagEvents["NiveaES"] = "UNIT_AURA"
oUF.TagEvents["NiveaRipT"] = "UNIT_AURA"
oUF.TagEvents["NiveaErLiv"] = "UNIT_AURA" 	

ns.tags = tags